@@ -224,7 +224,7 @@ class NodesTest(base.TestCase):
     def _get_node(self):
         return {'cpu': '1', 'memory': '2048', 'disk': '30', 'arch': 'amd64',
                 'mac': ['aaa'], 'pm_addr': 'foo.bar', 'pm_user': 'test',
-                'pm_password': 'random', 'pm_type': 'pxe_ssh', 'name': 'node1',
+                'pm_password': 'random', 'pm_type': 'ipmi', 'name': 'node1',
                 'capabilities': 'num_nics:6'}
 
     def test_register_all_nodes_ironic_no_hw_stats(self):
@@ -243,13 +243,13 @@ class NodesTest(base.TestCase):
         ironic = mock.MagicMock()
         new_nodes = nodes.register_all_nodes(node_list, client=ironic)
         self.assertEqual([ironic.node.create.return_value], new_nodes)
-        pxe_node_driver_info = {"ssh_address": "foo.bar",
-                                "ssh_username": "test",
-                                "ssh_key_contents": "random",
-                                "ssh_virt_type": "virsh"}
-        pxe_node = mock.call(driver="pxe_ssh",
+        pxe_node_driver_info = {"ipmi_address": "foo.bar",
+                                "ipmi_username": "test",
+                                "ipmi_password": "random"}
+        pxe_node = mock.call(driver="ipmi",
                              name='node1',
                              driver_info=pxe_node_driver_info,
+                             resource_class='baremetal',
                              properties=node_properties)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                               address='aaa')
@@ -265,13 +265,13 @@ class NodesTest(base.TestCase):
                            "capabilities": "num_nics:6"}
         ironic = mock.MagicMock()
         nodes.register_all_nodes(node_list, client=ironic)
-        pxe_node_driver_info = {"ssh_address": "foo.bar",
-                                "ssh_username": "test",
-                                "ssh_key_contents": "random",
-                                "ssh_virt_type": "virsh"}
-        pxe_node = mock.call(driver="pxe_ssh",
+        pxe_node_driver_info = {"ipmi_address": "foo.bar",
+                                "ipmi_username": "test",
+                                "ipmi_password": "random"}
+        pxe_node = mock.call(driver="ipmi",
                              name='node1',
                              driver_info=pxe_node_driver_info,
+                             resource_class='baremetal',
                              properties=node_properties)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                               address='aaa')
@@ -293,15 +293,15 @@ class NodesTest(base.TestCase):
         nodes.register_all_nodes(node_list, client=ironic,
                                  glance_client=glance, kernel_name='bm-kernel',
                                  ramdisk_name='bm-ramdisk')
-        pxe_node_driver_info = {"ssh_address": "foo.bar",
-                                "ssh_username": "test",
-                                "ssh_key_contents": "random",
-                                "ssh_virt_type": "virsh",
+        pxe_node_driver_info = {"ipmi_address": "foo.bar",
+                                "ipmi_username": "test",
+                                "ipmi_password": "random",
                                 "deploy_kernel": "kernel-123",
                                 "deploy_ramdisk": "ramdisk-999"}
-        pxe_node = mock.call(driver="pxe_ssh",
+        pxe_node = mock.call(driver="ipmi",
                              name='node1',
                              driver_info=pxe_node_driver_info,
+                             resource_class='baremetal',
                              properties=node_properties)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                               address='aaa')
@@ -318,14 +318,14 @@ class NodesTest(base.TestCase):
                            "capabilities": "num_nics:6"}
         ironic = mock.MagicMock()
         nodes.register_all_nodes(node_list, client=ironic)
-        pxe_node_driver_info = {"ssh_address": "foo.bar",
-                                "ssh_username": "test",
-                                "ssh_key_contents": "random",
-                                "ssh_virt_type": "virsh"}
-        pxe_node = mock.call(driver="pxe_ssh",
+        pxe_node_driver_info = {"ipmi_address": "foo.bar",
+                                "ipmi_username": "test",
+                                "ipmi_password": "random"}
+        pxe_node = mock.call(driver="ipmi",
                              name='node1',
                              driver_info=pxe_node_driver_info,
                              properties=node_properties,
+                             resource_class='baremetal',
                              uuid="abcdef")
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                               address='aaa')
@@ -344,36 +344,84 @@ class NodesTest(base.TestCase):
                            "capabilities": "num_nics:7"}
         ironic = mock.MagicMock()
         nodes.register_all_nodes(node_list, client=ironic)
-        pxe_node_driver_info = {"ssh_address": "foo.bar",
-                                "ssh_username": "test",
-                                "ssh_key_contents": "random",
-                                "ssh_virt_type": "virsh"}
-        pxe_node = mock.call(driver="pxe_ssh",
+        pxe_node_driver_info = {"ipmi_address": "foo.bar",
+                                "ipmi_username": "test",
+                                "ipmi_password": "random"}
+        pxe_node = mock.call(driver="ipmi",
                              name='node1',
                              driver_info=pxe_node_driver_info,
+                             resource_class='baremetal',
                              properties=node_properties)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                               address='aaa')
         ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
         ironic.port.create.assert_has_calls([port_call])
 
+    def test_register_all_nodes_with_interfaces(self):
+        interfaces = {'boot_interface': 'pxe',
+                      'console_interface': 'ipmitool-socat',
+                      'deploy_interface': 'direct',
+                      'inspect_interface': 'inspector',
+                      'management_interface': 'ipmitool',
+                      'network_interface': 'neutron',
+                      'power_interface': 'ipmitool',
+                      'raid_interface': 'agent',
+                      'storage_interface': 'cinder',
+                      'vendor_interface': 'ipmitool'}
+
+        node_list = [self._get_node()]
+        node_list[0].update(interfaces)
+        node_properties = {"cpus": "1",
+                           "memory_mb": "2048",
+                           "local_gb": "30",
+                           "cpu_arch": "amd64",
+                           "capabilities": "num_nics:6"}
+        ironic = mock.MagicMock()
+        nodes.register_all_nodes(node_list, client=ironic)
+        pxe_node_driver_info = {"ipmi_address": "foo.bar",
+                                "ipmi_username": "test",
+                                "ipmi_password": "random"}
+        pxe_node = mock.call(driver="ipmi",
+                             name='node1',
+                             driver_info=pxe_node_driver_info,
+                             properties=node_properties,
+                             resource_class='baremetal',
+                             **interfaces)
+        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
+                              address='aaa')
+        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
+        ironic.port.create.assert_has_calls([port_call])
+
     def test_register_update(self):
+        interfaces = {'boot_interface': 'pxe',
+                      'console_interface': 'ipmitool-socat',
+                      'deploy_interface': 'direct',
+                      'inspect_interface': 'inspector',
+                      'management_interface': 'ipmitool',
+                      'network_interface': 'neutron',
+                      'power_interface': 'ipmitool',
+                      'raid_interface': 'agent',
+                      'storage_interface': 'cinder',
+                      'vendor_interface': 'ipmitool'}
+
         node = self._get_node()
+        node.update(interfaces)
         ironic = mock.MagicMock()
         node_map = {'mac': {'aaa': 1}}
 
         def side_effect(*args, **kwargs):
             update_patch = [
                 {'path': '/name', 'value': 'node1'},
-                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
-                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
+                {'path': '/driver_info/ipmi_password', 'value': 'random'},
+                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                 {'path': '/properties/memory_mb', 'value': '2048'},
                 {'path': '/properties/local_gb', 'value': '30'},
                 {'path': '/properties/cpu_arch', 'value': 'amd64'},
                 {'path': '/properties/cpus', 'value': '1'},
                 {'path': '/properties/capabilities', 'value': 'num_nics:6'},
-                {'path': '/driver_info/ssh_username', 'value': 'test'},
-                {'path': '/driver_info/ssh_virt_type', 'value': 'virsh'}]
+                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
+            for iface, value in interfaces.items():
+                update_patch.append({'path': '/%s' % iface, 'value': value})
             for key in update_patch:
                 key['op'] = 'add'
             self.assertThat(update_patch,
@@ -395,8 +443,8 @@ class NodesTest(base.TestCase):
         def side_effect(*args, **kwargs):
             update_patch = [
                 {'path': '/name', 'value': 'node1'},
-                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
-                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
+                {'path': '/driver_info/ipmi_password', 'value': 'random'},
+                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                 {'path': '/properties/memory_mb', 'value': '2048'},
                 {'path': '/properties/local_gb', 'value': '30'},
                 {'path': '/properties/cpu_arch', 'value': 'amd64'},
@@ -404,8 +452,34 @@ class NodesTest(base.TestCase):
                 {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                 {'path': '/driver_info/deploy_kernel', 'value': 'image-k'},
                 {'path': '/driver_info/deploy_ramdisk', 'value': 'image-r'},
-                {'path': '/driver_info/ssh_username', 'value': 'test'},
-                {'path': '/driver_info/ssh_virt_type', 'value': 'virsh'}]
+                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
+            for key in update_patch:
+                key['op'] = 'add'
+            self.assertThat(update_patch,
+                            matchers.MatchesSetwise(*(map(matchers.Equals,
+                                                          args[1]))))
+            return mock.Mock(uuid='uuid1')
+
+        ironic.node.update.side_effect = side_effect
+        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
+        ironic.node.update.assert_called_once_with(1, mock.ANY)
+
+    def test_register_update_with_interfaces(self):
+        node = self._get_node()
+        ironic = mock.MagicMock()
+        node_map = {'mac': {'aaa': 1}}
+
+        def side_effect(*args, **kwargs):
+            update_patch = [
+                {'path': '/name', 'value': 'node1'},
+                {'path': '/driver_info/ipmi_password', 'value': 'random'},
+                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
+                {'path': '/properties/memory_mb', 'value': '2048'},
+                {'path': '/properties/local_gb', 'value': '30'},
+                {'path': '/properties/cpu_arch', 'value': 'amd64'},
+                {'path': '/properties/cpus', 'value': '1'},
+                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
+                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
             for key in update_patch:
                 key['op'] = 'add'
             self.assertThat(update_patch,
@@ -462,15 +536,14 @@ class NodesTest(base.TestCase):
         def side_effect(*args, **kwargs):
             update_patch = [
                 {'path': '/name', 'value': 'node1'},
-                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
-                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
+                {'path': '/driver_info/ipmi_password', 'value': 'random'},
+                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                 {'path': '/properties/memory_mb', 'value': '2048'},
                 {'path': '/properties/local_gb', 'value': '30'},
                 {'path': '/properties/cpu_arch', 'value': 'amd64'},
                 {'path': '/properties/cpus', 'value': '1'},
                 {'path': '/properties/capabilities', 'value': 'num_nics:6'},
-                {'path': '/driver_info/ssh_username', 'value': 'test'},
-                {'path': '/driver_info/ssh_virt_type', 'value': 'virsh'}]
+                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
             for key in update_patch:
                 key['op'] = 'add'
             self.assertThat(update_patch,
@@ -491,15 +564,14 @@ class NodesTest(base.TestCase):
         def side_effect(*args, **kwargs):
             update_patch = [
                 {'path': '/name', 'value': 'node1'},
-                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
-                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
+                {'path': '/driver_info/ipmi_password', 'value': 'random'},
+                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                 {'path': '/properties/memory_mb', 'value': '2048'},
                 {'path': '/properties/local_gb', 'value': '30'},
                 {'path': '/properties/cpu_arch', 'value': 'amd64'},
                 {'path': '/properties/cpus', 'value': '1'},
                 {'path': '/properties/capabilities', 'value': 'num_nics:6'},
-                {'path': '/driver_info/ssh_username', 'value': 'test'},
-                {'path': '/driver_info/ssh_virt_type', 'value': 'virsh'}]
+                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
             for key in update_patch:
                 key['op'] = 'add'
             self.assertThat(update_patch,
@@ -526,6 +598,7 @@ class NodesTest(base.TestCase):
         client.node.create.assert_called_once_with(driver=mock.ANY,
                                                    name='node1',
                                                    properties=node_properties,
+                                                   resource_class='baremetal',
                                                    driver_info=mock.ANY)
 
     def test_register_ironic_node_fake_pxe(self):
@@ -543,6 +616,7 @@ class NodesTest(base.TestCase):
         client.node.create.assert_called_once_with(driver='fake_pxe',
                                                    name='node1',
                                                    properties=node_properties,
+                                                   resource_class='baremetal',
                                                    driver_info={})
 
     def test_register_ironic_node_pxe_ucs(self):
@@ -557,6 +631,7 @@ class NodesTest(base.TestCase):
         nodes.register_ironic_node(node, client=client)
         client.node.create.assert_called_once_with(
             driver='pxe_ucs', name='node1', properties=node_properties,
+            resource_class='baremetal',
             driver_info={'ucs_password': 'random', 'ucs_address': 'foo.bar',
                          'ucs_username': 'test'})
 
@@ -573,6 +648,7 @@ class NodesTest(base.TestCase):
         nodes.register_ironic_node(node, client=client)
         client.node.create.assert_called_once_with(
             driver='ipmi', name='node1', properties=node_properties,
+            resource_class='baremetal',
             driver_info={'ipmi_password': 'random', 'ipmi_address': 'foo.bar',
                          'ipmi_username': 'test', 'ipmi_port': '6230'})
 
@@ -589,6 +665,7 @@ class NodesTest(base.TestCase):
         nodes.register_ironic_node(node, client=client)
         client.node.create.assert_called_once_with(
             driver='pxe_ipmitool', name='node1', properties=node_properties,
+            resource_class='baremetal',
             driver_info={'ipmi_password': 'random', 'ipmi_address': 'foo.bar',
                          'ipmi_username': 'test', 'ipmi_port': '6230'})
 
@@ -605,6 +682,7 @@ class NodesTest(base.TestCase):
         nodes.register_ironic_node(node, client=client)
         client.node.create.assert_called_once_with(
             driver='pxe_drac', name='node1', properties=node_properties,
+            resource_class='baremetal',
             driver_info={'drac_password': 'random', 'drac_address': 'foo.bar',
                          'drac_username': 'test', 'drac_port': '6230'})
 
@@ -621,6 +699,7 @@ class NodesTest(base.TestCase):
         nodes.register_ironic_node(node, client=client)
         client.node.create.assert_called_once_with(
             driver='redfish', name='node1', properties=node_properties,
+            resource_class='baremetal',
             driver_info={'redfish_password': 'random',
                          'redfish_address': 'foo.bar',
                          'redfish_username': 'test',
@@ -637,15 +716,14 @@ class NodesTest(base.TestCase):
         def side_effect(*args, **kwargs):
             update_patch = [
                 {'path': '/name', 'value': 'node1'},
-                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
-                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
+                {'path': '/driver_info/ipmi_password', 'value': 'random'},
+                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                 {'path': '/properties/memory_mb', 'value': '2048'},
                 {'path': '/properties/local_gb', 'value': '30'},
                 {'path': '/properties/cpu_arch', 'value': 'amd64'},
                 {'path': '/properties/cpus', 'value': '1'},
                 {'path': '/properties/capabilities', 'value': 'num_nics:6'},
-                {'path': '/driver_info/ssh_username', 'value': 'test'},
-                {'path': '/driver_info/ssh_virt_type', 'value': 'virsh'}]
+                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
             for key in update_patch:
                 key['op'] = 'add'
             self.assertThat(update_patch,
